#!/bin/sh

###############################################################
#   This script should be hooked when invoking SYNOUserSet()
#   Usable environment variable:
#       NITEMS, ORIGIN_USER_NAME_{n}, USER_NAME_{n}, UID_{n},
#		USER_OP_RESULT_{n}, RESULT  
#
#   Note: The script should bear to be invoked multiple times 
###############################################################

#Include common scripts
. /var/packages/CloudStation/target/scripts/CloudStationCommon

#At begining, acquire package settings
PackageInfoGet;

SYNO_USER_SET_ACT="user set"

move_user_repo()
{
	local from_name="$1"
	local to_name="$2"
	local volid=`get_share_vid homes`
	local vol_path="/volume${volid}"
	local repo_loc="${vol_path}/@cloudstation"
	local repo_client="${repo_loc}/@sync-client"
	local fp_from="${repo_client}/${from_name}"
	local fp_to="${repo_client}/${to_name}"
	
	if [ -d "${fp_from}" ]; then
		mv "${fp_from}" "${fp_to}"
	fi
}

case $1 in
    --sdk-mod-ver)
		#Print SDK support version
		echo "1.0";
    	;;
    --name)
		#Print package name
		echo ${CSTN_PKG_NAME};
    	;;
    --pkg-ver)
		#Print package version
		echo ${CSTN_PKG_VERSION};
    	;;
    --vendor)
		#Print package vendor
		echo ${CSTN_PKG_VENDOR};
    	;;

    --pre)
    	
    	is_cstn_upgrade_process_done;    	
    	if [ 0 -eq $? ]; then
			exit 0;
        fi
        
		#noop
		local usr_idx=1;
        local usr_cnt=$(($NITEMS + 1));

        while [ "$usr_idx" != "$usr_cnt" ]
        do
            eval "ori_name=\$ORIGIN_USER_NAME_$usr_idx"
			eval "new_name=\$USER_NAME_$usr_idx"
			eval "op_result=\$USER_OP_RESULT_$usr_idx"
			eval "usr_enabled=\$EXPIRE_STATE_$usr_idx"

			${CSTN_USRCTL_BIN} -s 0 "$ori_name" 0 0 > /dev/null 2>&1
            usr_idx=$(($usr_idx + 1));
        done
	
    	;;
    --post)
    
    	is_cstn_upgrade_process_done;    	
    	if [ 0 -eq $? ]; then
			exit 0;
        fi
        
		local usr_idx=1;
        local usr_cnt=$(($NITEMS + 1));

        while [ "$usr_idx" != "$usr_cnt" ]
        do
            eval "ori_name=\$ORIGIN_USER_NAME_$usr_idx"
			eval "new_name=\$USER_NAME_$usr_idx"
			eval "op_result=\$USER_OP_RESULT_$usr_idx"
			eval "usr_enabled=\$EXPIRE_STATE_$usr_idx"

			InfoMsg "$SYNO_USER_SET_ACT" "User expire stat is $usr_enabled."
		
			# Start user
			if [ "0x0000" = "$op_result" -a -n "${ori_name}" ]; then
				if [ "${ori_name}" = "${new_name}" ]; then
					${CSTN_USRCTL_BIN} -s 0 "$ori_name" 1 0
				else
					move_user_repo "${ori_name}" "${new_name}"
					${CSTN_USRCTL_BIN} -r "${ori_name}" "${new_name}"
					${CSTN_USRCTL_BIN} -s 0 "${new_name}" 1 0
				fi
			fi

            usr_idx=$(($usr_idx + 1));
        done

		touch "${CSTN_RELOAD_USERDB_CACHE_MARK}"

    	;;
    *)
		echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
    	;;
esac

