#!/bin/sh

#Conf dir/file definition
PKG_NAME="cloudstation"
PKG_FOLDER="/var/packages/CloudStation"
PKG_DIR="$PKG_FOLDER/target"
CONF_DIR="${PKG_FOLDER}/etc"
SYS_DIR_NAME="@${PKG_NAME}"

CONF_PATH="${CONF_DIR}/setting.conf"
PID_PATH="/var/run/synosyncfolder.pid"
CLIENT_DB_SQL="${PKG_DIR}/etc/event-db.sql"
SYNCD_PATH="${PKG_DIR}/sbin/syncd"
INDEXD_PATH="${PKG_DIR}/sbin/cloud-indexd"
USR_DB_PATH="${CONF_DIR}/user-db.sqlite"

USR_CTL_BIN="$PKG_DIR/bin/usrctl"

CSTN_DB_CONF="db-path.conf"
CSTN_SET_VAL_TOOL="/usr/syno/bin/synosetkeyvalue"


#Global variables
CSTN_ENABLE="0"
CSRV_ENABLE="0"
KEEP_HISTORY="onemonth"
KEEP_VER="10"
MAX_SIZE="256mb"
CSTN_PORT="6690"
CSTN_DBG_LEVEL=0

debug_msg() {
	#echo "CS: $1";
	if [ 2 -le $CSTN_DBG_LEVEL ]; then
		logger -p 0 "CS: [start.sh]: $1"
	fi
}

info_msg() {
	#echo "CS: $1";
	if [ 1 -le $CSTN_DBG_LEVEL ]; then
		logger -p 0 "CS: [start.sh]: $1"
	fi
}

err_msg() {
	if [ 0 -le $CSTN_DBG_LEVEL ]; then
		logger -p 0 "CS: [start.sh]: $1"
	fi
}

get_conf() {
	if [ -f "${CONF_PATH}" ]; then
    	CSTN_ENABLE=`/bin/get_key_value ${CONF_PATH} cstn_enable`
		CSRV_ENABLE=`/bin/get_key_value ${CONF_PATH} csrv_enable`
		KEEP_HISTORY=`/bin/get_key_value ${CONF_PATH} keep_history`
		KEEP_VER=`/bin/get_key_value ${CONF_PATH} keep_ver`
		MAX_SIZE=`/bin/get_key_value ${CONF_PATH} max_size`
		CSTN_PORT=`/bin/get_key_value ${CONF_PATH} cstn_port`
    else
    	err_msg "Cloud station setting config does not exist!";
    	exit 1;
    fi
}

is_daemon_alive() {
    if [ -f "$1" ]; then
		local pid=`cat "$1"`
		
		kill -0 $pid
		if [ "0" = "$?" ]; then
	    	echo "$pid";
			return 1;
		else
			echo "0";
			return 0;
		fi
    fi
	
	echo "0";
	return 0;
}

stop_daemon() {
	${USR_CTL_BIN} -s 1 "dummy" 0 0 > /dev/null 2>&1
    if [ "0" != "$?" ]; then
        err_msg "Fail to disable user!"
    fi

	local pid=`is_daemon_alive "$1"`
	if [ "0" != "$pid" ]; then
		kill -15 ${pid}
	fi

	while [ "0" != `is_daemon_alive "$1"` ]
	do
		sleep 1;
	done

	# stop cloud-indexd
	kill `cat /var/run/cloud-indexd.pid`

}

get_homes_vol() {
	local homes_vol=`synoshare --get homes | grep "Path ......." | sed -e 's/^.*\[\(.*\)\/.*\]/\1/g'`;
	
	if [ "x" = "x${homes_vol}" ]; then
		err_msg "Fail to get homes volume!";
		exit 1;
	fi

	echo "${homes_vol}"
	return 0;
}


prepare_env() {
	local vol=`get_homes_vol`
	local conf_svr_dir="$vol/$SYS_DIR_NAME/@sync"
	local conf_client_dir="$vol/$SYS_DIR_NAME/@sync-client"
	local conf_share_dir="$vol/$SYS_DIR_NAME/@cstn-share"
	local repo="${conf_svr_dir}/repo"
	local conf="${conf_svr_dir}/db.conf"
	local db_loc="${conf_svr_dir}/syncfolder-db.sqlite"

	# if conf file doesn't exist, then we have no choice to deem current home vol as db path
	# when remove repo ?
	# 1. if path conf does not exist
	# 2. if path conf exists but differ with current home volume
	local remove_repo=0
	if [ ! -f "$CONF_DIR/$CSTN_DB_CONF" ]; then
		logger -p 0 " path conf file: $CONF_DIR/$CSTN_DB_CONF does not exist !"
		remove_repo=1
	else
		local ORIG_VOL=`/bin/get_key_value "$CONF_DIR/$CSTN_DB_CONF" db-vol`
		if [ "$vol" != "$ORIG_VOL" ]; then
			logger -p 0 "$vol and $ORIG_VOL different !"
			remove_repo=1
		fi
	fi
	
	if [ $remove_repo -eq 1 ]; then
		logger -p 0 "remove $vol/$SYS_DIR_NAME"
		if [ -d "$vol/$SYS_DIR_NAME" ]; then
			local tmp_path="$vol/@tmp/$SYS_DIR_NAME.$$"
			mkdir -p "$tmp_path"
			mv -f "$vol/$SYS_DIR_NAME" "$tmp_path"
			rm -rf "$tmp_path" &
		fi
	fi

	logger -p 0 "calling  $CSTN_SET_VAL_TOOL $CONF_DIR/$CSTN_DB_CONF db-vol $vol"
	
	$CSTN_SET_VAL_TOOL "$CONF_DIR/$CSTN_DB_CONF" db-vol "$vol"

	if [ ! -d "$repo" ]; then
		mkdir -p "$repo";
		chmod 777 "$vol/$SYS_DIR_NAME"
		chmod 777 "$conf_svr_dir"
	fi

	if [ ! -d "$conf_client_dir" ]; then
		mkdir -p "$conf_client_dir"
		chmod 777 "$conf_client_dir"
	fi

	if [ ! -d "$conf_share_dir" ]; then
		mkdir -p "$conf_share_dir"
		chmod 777 "$conf_share_dir"
	fi
	
}

remove_env() {
	local vol=`get_homes_vol`
	local conf_svr_dir="$vol/$SYS_DIR_NAME/@sync"
	local conf_client_dir="$vol/$SYS_DIR_NAME/@sync-client"
	local repo="${conf_svr_dir}/repo"
	local conf="${conf_svr_dir}/db.conf"
	local db_loc="${conf_svr_dir}/syncfolder-db.sqlite"
	local usr_db="${conf_svr_dir}/user-db.sqlite"

	if [ -d "${conf_svr_dir}" ]; then
		rm -rf "${conf_svr_dir}";
	fi

	if [ -d "${conf_client_dir}" ]; then
		rm -rf "${conf_client_dir}";
	fi
}


start_daemon() {
	local vol=`get_homes_vol`
	local conf_dir="$vol/$SYS_DIR_NAME/@sync"
	local repo="${conf_dir}/repo"
	local conf="${conf_dir}/db.conf"
	local db_loc="${conf_dir}/syncfolder-db.sqlite"
	local usr_db="${conf_dir}/user-db.sqlite"
	local is_first=0

	if [ ! -f "${usr_db}" ]; then
		is_first=1
	fi

	#check log folder
	if [ ! -d "/var/log/cstn" ]; then
		mkdir -p "/var/log/cstn"
		chmod 777 "/var/log/cstn"
	fi

	# start cloud-indexd
	${INDEXD_PATH}

	${SYNCD_PATH} -c "${db_loc}" -u "${usr_db}" -p "${CSTN_PORT}"
	if [ 1 -eq $is_first ]; then
		${USR_CTL_BIN} -s 0 "admin" 1 1 > /dev/null 2>&1
		/usr/syno/sbin/synorelayd -a cloudstation > /dev/null 2>&1
	else
		${USR_CTL_BIN} -s 1 "dummy" 1 0 > /dev/null 2>&1
	fi

	if [ "0" != "$?" ]; then
		err_msg "Fail to enable user!"
	fi

}

USER_HOME_ENABLE=`/bin/get_key_value /etc/synoinfo.conf userHomeEnable`
if [ "x${USER_HOME_ENABLE}" != "xyes" ]; then
	err_msg "CS: You must enable user home first!"
	exit 1;
fi

get_conf;

case $1 in
    start)
		err_msg "Cloud station start..."

		#Check if server is enable
		if [ "1" == "${CSTN_ENABLE}" ]; then
			local pid=`is_daemon_alive ${PID_PATH}`
			if [ "0" != "$pid" ]; then
				err_msg "Cloud station daemon exists!";
				exit 1;
			fi

			prepare_env;
			start_daemon;
		else
	    	exit 0;
		fi
		;;

    stop)
		err_msg "Cloud station stop..."
		
#		if [ "1" != "${CSTN_ENABLE}" ]; then
#			remove_env;		
#		fi
		stop_daemon "${PID_PATH}"
   		;;

    restart)
		debug_msg "Cloud station restart..."
		$0 stop;
		$0 start;
    	;;
    *)
		echo "Usage: $0 start|stop|restart"
    ;;
esac

